import random
import tensorflow as tf
import tensorflow.keras as keras
import numpy as np
import matplotlib.pyplot as plt
import tensorflow.keras.backend as K
import os

fileDir = os.path.dirname(os.path.abspath(__file__))

#instruments = 14
#trackLength = 32


# Prints all sets in the given array
def printAllBin(data):
    counter = 1
    for set in data:
        print("Set Nr: " + str(counter))
        printSetBin(set)
        counter += 1

# Prints the set in binary representation
def printSetBin(set):
    counter = 0
    
    for rowint in set:
        rowint = int(rowint)
        line = ""
        for div in range(32):
            line = str(int(rowint%2))+","+line
            rowint = int(rowint/2)
        print(line)

# Prints the set in integer representation
def printSetInt(set):
    print("next Set:")
    for inte in set:
        print(str(int(inte)))
    print("\n")

# Prints all sets in the given array in integer-per row presentation
def printAllInt(data):
    for set in data:
        printSetInt(set)    

sets=[]
results=[]

# Load data from testfile
path = os.path.join(fileDir, "testdata.txt")
dataFile = open(path,"r")
print ("Reading from : " + path)
dataString=dataFile.read()
dataPairs=dataString.split("\n")
it = iter(dataPairs)
for line in it:
    if(line != ""):
        patternString = line
        coordsString = next(it)
        results.append([float(i) for i in coordsString.split(",")])
        sets.append([float(i) for i in patternString.split(",")[:-1]])
print("First Data Sample: ")
print(sets[0])
print (results[0])

actFnc='relu'

#define model
model = keras.Sequential([
    keras.layers.Dense(units=4, activation=actFnc, input_shape=[4], name="Input_Layer"),
    keras.layers.Dense(128, activation=actFnc,kernel_initializer='random_normal'),
    keras.layers.Dense(256, activation=actFnc,kernel_initializer='random_normal'),
    keras.layers.Dense(units=8,activation=actFnc,kernel_initializer='random_normal')
])

model.compile(optimizer='adam', loss='mean_absolute_error', metrics=['acc'])

# train the model using the generated data
history = model.fit(x=tf.convert_to_tensor(results),y=tf.convert_to_tensor(sets),epochs=1000)

print(model.summary())

#Test the model with a couple of datasets
tests = model.predict(tf.convert_to_tensor([[2.0,2.54,1.0,0.4], [-2.0,2.54,-2.0,2.54], [-2.0,-2.54,2.54,1.0],[-0.49095791578292847,-0.11129298061132431,0.4223683476448059,0.5134237408638],[1.2648082971572876,1.4164128303527832,0.7757405638694763,0.8534218072891235],[0.45829272270202637,-1.768419623374939,0.547592282295227,1.737666130065918],[-0.10240457952022552,1.012917160987854,-2.0220065116882324,0.8447896242141724],[-0.5929307341575623,1.9832522869110107,-0.5629125237464905,1.0216374397277832]]))
printAllBin(tests)
printAllInt(tests)
print(history.history.keys())

print("Finished model fitting using " + str(len(results)) + " input sets")

# plot the history
#accuracy
plt.plot(history.history['acc'])
plt.title('model acc')
plt.ylabel('acc')
plt.xlabel('epoch')
plt.legend(['train','test'], loc='upper left')
plt.show()

#loss
plt.plot(history.history['loss'])
plt.title('model loss')
plt.ylabel('loss')
plt.xlabel('epoch')
plt.legend(['train','test'], loc='upper left')
plt.show()#

#create and save tflite model
converter = tf.lite.TFLiteConverter.from_keras_model(model)
tflite_model = converter.convert()

open("8_Int_Flat_Latent.tflite", "wb").write(tflite_model)