import random
import tensorflow as tf
import tensorflow.keras as keras
import numpy as np
import matplotlib.pyplot as plt
import tensorflow.keras.backend as K

#instruments = 14
#trackLength = 32


# Prints all sets in the given array
def printAll(data):
    counter = 1
    for set in data:
        print("Set Nr: " + str(counter))
        printSet(set)
        counter += 1

# Prints the set
def printSet(set):
    counter = 0
    line =""
    for step in set:
        counter = counter + 1
        line += str(int(step)) + "|"
        if(counter%32 == 0):
            line += "\n"
    print(line)

sets=[]
results=[]

# Load data from testfile
dataFile = open("./Boolean_Flat_latent/testdata.txt","r")
dataString=dataFile.read()
dataPairs=dataString.split("\n")
it = iter(dataPairs)
for line in it:
    if(line != ""):
        patternString = line
        coordsString = next(it)
        results.append([float(i) for i in coordsString.split(",")])
        sets.append([float(i) for i in patternString.split(",")[:-1]])

model = keras.Sequential([
    keras.layers.Dense(units=4, activation='relu', input_shape=[4], name="Input_Layer"),
    keras.layers.Dense(256, activation='relu',kernel_initializer=tf.keras.initializers.RandomNormal(mean=0.5, stddev=0.25, seed=None)),
    keras.layers.Dense(units=448,activation='relu',kernel_initializer=tf.keras.initializers.RandomNormal(mean=0.5, stddev=0.25, seed=None))
])

model.compile(optimizer='adam', loss=tf.keras.losses.MeanSquaredError(), metrics=['acc'])
K.set_value(model.optimizer.learning_rate, 0.001)

# train the model using the generated data
history = model.fit(x=tf.convert_to_tensor(results),y=tf.convert_to_tensor(sets),epochs=100)

model.summary()

#Test the model with a couple of datasets
tests = model.predict(tf.convert_to_tensor([[2.0,3.0,1.0,0.4], [-2.0,3.0,-2.0,3.0], [-2.0,-3.0,3.0,1.0]]))
printAll(tests)
print(history.history.keys())

print("Finished model fitting using" + str(len(results)) + " input sets")

# plot the history
# accuracy
plt.plot(history.history['acc'])
plt.title('model acc')
plt.ylabel('acc')
plt.xlabel('epoch')
plt.legend(['train','test'], loc='upper left')
plt.show()

# loss
plt.plot(history.history['loss'])
plt.title('model loss')
plt.ylabel('loss')
plt.xlabel('epoch')
plt.legend(['train','test'], loc='upper left')
plt.show()

# create and save tflite model
converter = tf.lite.TFLiteConverter.from_keras_model(model)
tflite_model = converter.convert()

open("flat_bool_latent.tflite", "wb").write(tflite_model)