from operator import truediv
import random
import tensorflow as tf
import tensorflow.keras as keras
import numpy as np
import matplotlib.pyplot as plt
import tensorflow.keras.backend as K

import re

#instruments = 14
#trackLength = 32


# Prints all sets in the given array
def printAll(data):
    counter = 1
    for set in data:
        print("Set Nr: " + str(counter))
        printSet(set)
        counter += 1

# Prints a textual representation of the given set
def printSet(set):
    counter = 0
    line = ""
    for step in set:
        counter = counter + 1
        line += str(int(step)) + "|"
        if(counter % 32 == 0):
            line += "\n"
    print(line)

# Prints a textual representation of the given data
def printResult(res):
    matrix = []
    for col in res:
        newCol = []
        colInt = int(col)
        for i in range(0, 32):
            newCol.append(int(colInt % 2))
            colInt = colInt/2
        matrix.append(newCol)

    output = ""

    for row in range(1, 14):
        for col in range(1, 32):
            output = output + str(int(matrix[col][row])) + ","
        output = output + "\n"

    print(output)


sets = []
results = []

# Load data from testfile
dataFile = open("./dataset_raw.tsv", "r")
dataString = dataFile.read()
lines = dataString.split("\n")
printed = False
for line in lines:
    values = re.split(',|\t', line)
    coords = np.asfarray(values[32:36])
    ints = list(map(int, values[0:32]))
    sets.append(ints)
    results.append(coords)
    if not printed:
        print("ints:")
        print(ints)
        print("coords:")
        print(coords)
    printed = True

model = keras.Sequential([
    keras.layers.Dense(units=4, activation='linear',
                       input_shape=[4], name="Input_Layer"),
    keras.layers.Dense(16, activation='linear',
                       kernel_initializer='random_normal'),
    keras.layers.Dense(32, activation='linear',
                       kernel_initializer='random_normal'),
    keras.layers.Dense(64, activation='linear',
                       kernel_initializer='random_normal'),
    keras.layers.Dense(units=32, activation='linear',
                       kernel_initializer='random_normal')
])

model.compile(optimizer='adam',
              loss=tf.keras.losses.MeanSquaredError(), metrics=['acc'])

# train the model using the generated data
history = model.fit(x=tf.convert_to_tensor(results),
                    y=tf.convert_to_tensor(sets), epochs=1000)

model.summary()

# Test the model with a couple of datasets
tests = model.predict(tf.convert_to_tensor(
    [[2.0, 3.0, 1.0, 0.4], [-2.0, 3.0, -2.0, 3.0], [-2.0, -3.0, 3.0, 1.0]]))
for res in tests:
    printResult(res)

print("Finished model fitting using" + str(len(results)) + " input sets")

# plot the history
# accuracy
plt.plot(history.history['acc'])
plt.title('model acc')
plt.ylabel('acc')
plt.xlabel('epoch')
plt.legend(['train', 'test'], loc='upper left')
plt.show()

# loss
plt.plot(history.history['loss'])
plt.title('model loss')
plt.ylabel('loss')
plt.xlabel('epoch')
plt.legend(['train', 'test'], loc='upper left')
plt.show()

converter = tf.lite.TFLiteConverter.from_keras_model(model)
tflite_model = converter.convert()

tests= model.predict(tf.convert_to_tensor(
    [[2.0, 3.0, 1.0, 0.4], [-2.0, 3.0, -2.0, 3.0], [-2.0, -3.0, 3.0, 1.0]]))
for res in tests:
    printResult(res)